import ezomero


def test_connect_params(omero_params, tmp_path, monkeypatch):
    user, password, host, port, secure = omero_params

    # params should override environment variables
    monkeypatch.setenv("OMERO_USER", 'fail')
    monkeypatch.setenv("OMERO_PASS", 'fail')
    monkeypatch.setenv("OMERO_GROUP", 'fail')
    monkeypatch.setenv("OMERO_HOST", 'fail')
    monkeypatch.setenv("OMERO_PORT", 'fail')
    monkeypatch.setenv("OMERO_SECURE", 'fail')

    # params should override config file
    conf_txt = ("[DEFAULT]\n"
                "omero_user = fail\n"
                "omero_group = fail\n"
                "omero_host = fail\n"
                "omero_port = 9999\n"
                "omero_secure = True\n")
    conf_path = tmp_path / '.ezomero'
    conf_path.write_text(conf_txt)

    conn = ezomero.connect(user, password, host=host, group='', port=port,
                           secure=True, config_path=str(tmp_path))
    assert conn.getUser().getName() == user
    conn.close()


def test_connect_env(omero_params, tmp_path, monkeypatch):
    user, password, host, port, secure = omero_params
    monkeypatch.setenv("OMERO_USER", user)
    monkeypatch.setenv("OMERO_PASS", password)
    monkeypatch.setenv("OMERO_HOST", host)
    monkeypatch.setenv("OMERO_PORT", port)
    monkeypatch.setenv("OMERO_SECURE", 'True')

    # env should override config file
    conf_txt = ("[DEFAULT]\n"
                "omero_user = fail\n"
                "omero_group = fail\n"
                "omero_host = fail\n"
                "omero_port = 9999\n"
                "omero_secure = True\n")
    conf_path = tmp_path / '.ezomero'
    conf_path.write_text(conf_txt)

    conn = ezomero.connect(group='', config_path=str(tmp_path))
    assert conn.getUser().getName() == user
    conn.close()


def test_connect_config(omero_params, tmp_path):
    user, password, host, port, secure = omero_params
    conf_txt = ("[DEFAULT]\n"
                f"omero_user = {user}\n"
                "omero_group = \n"
                f"omero_host = {host}\n"
                f"omero_port = {port}\n"
                "omero_secure = True\n")
    conf_path = tmp_path / '.ezomero'
    conf_path.write_text(conf_txt)
    conn = ezomero.connect(password=password, config_path=str(tmp_path))
    assert conn.getUser().getName() == user
    conn.close()


def test_store_conn_params(omero_params, tmp_path):
    user, password, host, port, secure = omero_params
    ezomero.store_connection_params(user=user, group="", host=host, port=port,
                                    secure=True, config_path=str(tmp_path))
    conn = ezomero.connect(password=password, config_path=str(tmp_path))
    assert conn.getUser().getName() == user
    conn.close()



import ezomero
from omero.gateway import TagAnnotationWrapper


def test_filter_by_tag_value(conn, project_structure, users_groups):
    proj3_id = project_structure[0][3][1]
    im3_id = project_structure[2][3][1]
    group2_id = users_groups[0][1][1]

    current_group = conn.getGroupFromContext().getId()
    conn.SERVICE_OPTS.setOmeroGroup(group2_id)
    tag_ann = TagAnnotationWrapper(conn)
    tag_ann.setValue('test_tag_filter')
    tag_ann.save()
    tag_id = tag_ann.getId()

    im = conn.getObject('Image', im3_id)
    im.linkAnnotation(tag_ann)

    proj3_all_im_ids = ezomero.get_image_ids(conn, project=proj3_id)

    proj3_tag_only_im_ids = ezomero.filter_by_tag_value(conn, proj3_all_im_ids,
                                                        'test_tag_filter')

    assert set(proj3_tag_only_im_ids) == set([im3_id])

    conn.deleteObjects("Annotation",
                       [tag_id],
                       deleteAnns=True,
                       deleteChildren=True,
                       wait=True)
    conn.SERVICE_OPTS.setOmeroGroup(current_group)


def test_filter_by_kvpair(conn, project_structure):
    proj3_id = project_structure[0][3][1]
    im3_id = project_structure[2][3][1]

    suconn = conn.suConn('test_user1', 'test_group_2')

    kv_dict = {'testkey': 'testvalue',
               'testkey2': 'testvalue2'}

    ma_id = ezomero.post_map_annotation(suconn, 'Image', im3_id,
                                        kv_dict=kv_dict, ns='test')

    proj3_all_im_ids = ezomero.get_image_ids(suconn, project=proj3_id)

    proj3_kv_only_im_ids = ezomero.filter_by_kv(suconn,
                                                proj3_all_im_ids,
                                                key='testkey2',
                                                value='testvalue2')

    assert set(proj3_kv_only_im_ids) == set([im3_id])

    suconn.deleteObjects("Annotation",
                         [ma_id],
                         deleteAnns=True,
                         deleteChildren=True,
                         wait=True)
    suconn.close()


def test_prints(conn, project_structure):
    pid = project_structure[0][0][1]
    ezomero.print_datasets(conn, project=pid)
    ezomero.print_projects(conn)
    ezomero.print_groups(conn)



from dataclasses import dataclass, field
from typing import List, Tuple

__all__ = ["Point",
           "Line",
           "Rectangle",
           "Ellipse",
           "Polygon"]


@dataclass(frozen=True)
class Point:
    """A dataclass used to create an OMERO Point.

    A dataclass used to represent a Point shape and create an OMERO equivalent.
    This dataclass is frozen and should not be modified after instantiation

    Parameters
    ----------
    x : float
        The x axis position of the point shape in pixels.
    y : float
        The y axis position of the point shape in pixels.
    z : int, optional
        The z position of the point in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the Point will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the Point will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    """

    x: float = field(metadata={'units': 'PIXELS'})
    y: float = field(metadata={'units': 'PIXELS'})
    z: int = field(default=None)
    c: int = field(default=None)
    t: int = field(default=None)
    label: str = field(default=None)


@dataclass(frozen=True)
class Line:
    """A dataclass used to create an OMERO Line.

    A dataclass used to represent a Line shape and create an OMERO equivalent.
    This dataclass is frozen and should not be modified after instantiation.

    Parameters
    ----------
    x1 : float
        The x axis position of the start point of the line shape in pixels.
    y1 : float
        The y axis position of the start point of the line shape in pixels.
    x2 : float
        The x axis position of the end point of the line shape in pixels.
    y2 : float
        The y axis position of the end point of the line shape in pixels.
    z : int, optional
        The z position of the shape in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the shape will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the shape will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    """

    x1: float = field(metadata={'units': 'PIXELS'})
    y1: float = field(metadata={'units': 'PIXELS'})
    x2: float = field(metadata={'units': 'PIXELS'})
    y2: float = field(metadata={'units': 'PIXELS'})
    z: int = field(default=None)
    c: int = field(default=None)
    t: int = field(default=None)
    label: str = field(default=None)


@dataclass(frozen=True)
class Rectangle:
    """A dataclass used to create an OMERO rectangle.

    A dataclass used to represent a Rectangle shape and create an OMERO
    equivalent. This dataclass is frozen and should not be modified after
    instantiation.

    Parameters
    ----------
    x : float
        The x axis position of the rectangle shape in pixels.
    y : float
        The y axis position of the rectangle shape in pixels.
    width : float
        The width (x axis) of the rectangle shape in pixels.
    height : float
        The height (y axis) of the rectangle shape in pixels.
    z : int, optional
        The z position of the shape in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the shape will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the shape will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    """

    x: float = field(metadata={'units': 'PIXELS'})
    y: float = field(metadata={'units': 'PIXELS'})
    width: float = field(metadata={'units': 'PIXELS'})
    height: float = field(metadata={'units': 'PIXELS'})
    z: int = field(default=None)
    c: int = field(default=None)
    t: int = field(default=None)
    label: str = field(default=None)


@dataclass(frozen=True)
class Ellipse:
    """A dataclass used to create an OMERO Ellipse.

    A dataclass used to represent an Ellipse shape and create an OMERO
    equivalent. This dataclass is frozen and should not be modified after
    instantiation.

    Parameters
    ----------
    x : float
        The x axis position of the ellipse shape in pixels.
    y : float
        The y axis position of the ellipse shape in pixels.
    x_rad : float
        The x radius of the ellipse shape in pixels.
    y_rad : float
        The y radius of the ellipse shape in pixels.
    z : int, optional
        The z position of the shape in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the shape will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the shape will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    """

    x: float = field(metadata={'units': 'PIXELS'})
    y: float = field(metadata={'units': 'PIXELS'})
    x_rad: float = field(metadata={'units': 'PIXELS'})
    y_rad: float = field(metadata={'units': 'PIXELS'})
    z: int = field(default=None)
    c: int = field(default=None)
    t: int = field(default=None)
    label: str = field(default=None)


@dataclass(frozen=True)
class Polygon:
    """A dataclass used to create an OMERO polygon.

    A dataclass used to represent a Polygon shape and create an OMERO
    equivalent. This dataclass is frozen and should not be modified after
    instantiation.

    Parameters
    ----------
    points : list of tuples of 2 floats
        A list of 2 element tuples corresponding to the (x, y) coordinates of
        each vertex of the polygon.
    z : int, optional
        The z position of the shape in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the shape will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the shape will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    """

    points: List[Tuple[float, float]] = field(metadata={'units': 'PIXELS'})
    z: int = field(default=None)
    c: int = field(default=None)
    t: int = field(default=None)
    label: str = field(default=None)
